#!/usr/bin/perl
#===============================================================#
# netapp-ontapsdk-perf.pl                                       #
#                                                               #
# Usage of following APIs:                  			#
#                       perf-object-list-info                   #
#                       perf-object-counter-list-info           #
#                       perf-object-instance-list-info          #
#                       perf-object-get-instances               #
#                                                               #
# Provide Performances data from Netapp Filer using 		#
# Netapp Manage OnTap SDK 3.0 					#
# info: http://communities.netapp.com/docs/DOC-1110 		#
# script output is compatible for Cacti templates               #
#                                                               #
#                                                               #
# by:   Pierre-Luc Dion <pdion@expedia.com>                     #
# date: 2008-05-29                                              #
#                                                               #
#===============================================================#
# based on  perf_operation.pl from NetApp                       #
# ---------------------------------------                       #
# Copyright 2005 Network Appliance, Inc. All rights             #
# reserved. Specifications subject to change without notice.    #
#                                                               #
# This SDK sample code is provided AS IS, with no support or    #
# warranties of any kind, including but not limited to          #
# warranties of merchantability or fitness of any kind,         #
# expressed or implied.  This code is subject to the license    #
# agreement that accompanies the SDK.                           #
#                                                               #
#===============================================================#

$progname = "netapp-ontapsdk-perf.pl";

require 5.6.1;
use lib "/usr/local/lib/perl/NetApp";  	# Be sure this is the correct PATH
use NaServer;
use NaElement;

# Variable declaration
my $args = $#ARGV + 1;
my $filer = shift;
my $user = shift;
my $pw  = shift;
my $obj_name = shift;
my $command = shift;

#Invoke routine
main();

sub main() 
{
        # check for valid number of parameters
        if ($args < 4)
        {
                print_usage();
        }

        my $s = NaServer->new ($filer, 1, 3);

        $out = $s->set_transport_type(HTTP);
        if (ref ($out) eq "NaElement") { 
                if ($out->results_errno != 0) {
                        my $r = $out->results_reason();
                        print "Connection to $filer failed: $r\n";
                        exit (-2);
                }
        }

        $out = $s->set_style(LOGIN);
        if (ref ($out) eq "NaElement") { 
                if ($out->results_errno != 0) {
                        my $r = $out->results_reason();
                        print "Connection to $filer failed: $r\n";
                        exit (-2);
                }
        }

        $out = $s->set_admin_user($user, $pw);

        if($command eq "object-list")
        {
                get_object_list($s);
        }
        elsif($command eq "index")
        {
                get_instance_list($s);
        }
        elsif($command eq "counter-list")
        {
                get_counter_list($s);
        }
        elsif($command eq "query")
        {
                query_counter_values($s);
        }
        elsif($command eq "get")
        {
                get_counter_values($s);
        }

        else
        {
                print "Invalid operation\n";
                print_usage();
        }
        exit 0; 
}


# perf-object-list-info operation
# Usage: $progname <filer> <user> <password> object-list
sub get_object_list
{
        my $s = $_[0];
                
        if ($args < 4) 
        {
                print "Usage: $progname <filer> <user> <password> object-list \n";
                exit -1;
        }
        my $in = NaElement->new("perf-object-list-info");
        
         
        # Invoke API
        my $out = $s->invoke_elem($in);
        
        if($out->results_status() eq "failed")
        {
                print($out->results_reason() ."\n");
                exit(-2);
        }

        my $obj_info = $out->child_get("objects");
        my @result = $obj_info->children_get();
        
        foreach $obj (@result){
                my $obj_name = $obj->child_get_string("name");
                my $priv = $obj->child_get_string("privilege-level");
                print("Object Name = $obj_name \t\t Privilege Level = $priv\n");
        }
        print "\n";
}

# perf-object-instance-list-info operation
# Usage: $progname <filer> <user> <password> <objectname> index
 
sub get_instance_list
{
        my $s = $_[0];
        
        if ($args < 4) 
        {
                print "Usage: $progname <filer> <user> <password> <objectname> index\n";
                exit -1;
        }
        
        my $in = NaElement->new("perf-object-instance-list-info");
        $in->child_add_string("objectname",$obj_name);

        
        # Invoke API
        my $out = $s->invoke_elem($in);
                
        if($out->results_status() eq "failed")
        {
                print($out->results_reason() ."\n");
                exit(-2);
        }

        my $inst_info = $out->child_get("instances");
        my @result = $inst_info->children_get();
        foreach $inst (@result){
                my $inst_name = $inst->child_get_string("name");
		if ( $inst_name =~ m/:/ ) {             
			# This remove stuff after ":" in LUN name"
                        $inst_name =~ /(.*) :/;
                        $inst_name = $1;
                }
                print("$inst_name\n");
        }
}

# perf-object-counter-list-info operation
# Usage: $progname <filer> <user> <password> counter-list <objectname>
sub get_counter_list
{
        my $s = $_[0];
        
        if ($args < 4) 
        {
                print "Usage: $progname <filer> <user> <password> <objectname> get <counter> <instance>";
                exit -1;
        }
        
        my $in = NaElement->new("perf-object-counter-list-info");
        $in->child_add_string("objectname",$obj_name);  

        # Invoke API
        my $out = $s->invoke_elem($in);
                
        if($out->results_status() eq "failed")
        {
                print($out->results_reason() ."\n");
                exit(-2);
        }

        my $counter_info = $out->child_get("counters");
        my @result = $counter_info->children_get();
        foreach $counter (@result){
                my $counter_name = $counter->child_get_string("name");
                print("Counter Name = $counter_name \t\t\t");
                if ($counter->child_get_string("base-counter")) {
                        my $base_counter = $counter->child_get_string("base-counter");
                        print("Base Counter = $base_counter\t\t");
                } else {
                        print("Base Counter = none\t\t");
                }
                my $privilege_level = $counter->child_get_string("privilege-level");
                print("Privilege_level = $privilege_level\t\t");

                if ($counter->child_get_string("unit")) {
                        my $unit = $counter->child_get_string("unit");
                        print("Unit = $unit\t\t");
                } else {
                        print("Unit = none\t\t");
                }
                print "\n";
        }
        
}

# perf-object-get-instances operation
# Usage: $progname <filer> <user> <password> get-counter-values <objectname> <counter>
# output performance stats compatible for cacti usage. 
sub query_counter_values
{
        my $s = $_[0];
        my $out;
        my $in;
        my $counters;
        my $num_records;
        my $counter_selected; 

        if ($args < 6) 
        {
                print "Usage: $progname <filer> <user> <password> <objectname> query <counter>\n";
		print "Usage: $progname <filer> <user> <password> <objectname> query index\n";
                exit -1;
        }

        $counter_selected = @ARGV[0];
             
        $in = NaElement->new("perf-object-get-instances");
        $in->child_add_string("objectname", $obj_name);
        $out = $s->invoke_elem($in);

        if($out->results_status() eq "failed")
        {
                print($out->results_reason() ."\n");
                exit(-2);
        }

        my $instances_list = $out->child_get("instances");
        my @instances = $instances_list->children_get();

        foreach $inst (@instances){
                my $inst_name = $inst->child_get_string("name");
		if ( $inst_name =~ m/:/ ) {             
                        # This remove stuff after ":" in LUN name"
                        $inst_name =~ /(.*) :/;
                        $inst_name = $1;
                }
                my $counters_list = $inst->child_get("counters");
                my @counters =  $counters_list->children_get();
                if (! "$counter_selected") {
                        print_counter($inst_name, $inst_name);
                }
                if ($counter_selected eq "index")
                {
                        print ("$inst_name:$inst_name\n");
                }
                foreach $counter (@counters) 
                {
                        my $counter_name = $counter->child_get_string("name");
                        my $counter_value = $counter->child_get_string("value");
                        if ("$counter_name" eq "$counter_selected")
                        {
                                print ("$inst_name:$counter_value\n");
                        }
                }
        }
}

# perf-object-get-instances operation
# Usage: $progname <filer> <user> <password> <objectname> get <objectname> [<counter> <instance>]
# output performance stats compatible for cacti usage. 
# from counter to instance
sub get_counter_values
{
        my $s = $_[0];
        my $out;
        my $in;
        my $counters;
        my $num_records;
        my $instance_selected;
        my $counter_selected; 

        if ($args < 5) 
        {
                print "Usage: $progname <filer> <user> <password> <objectname> get [<counter> <instance>]\n";
		print "Usage: $progname netapp01 cacti <password> volume       get avg_latency vol0\n";
                exit -1;
        }
        
        $counter_selected = @ARGV[0];
        $instance_selected = @ARGV[1];
              
        $in = NaElement->new("perf-object-get-instances");
        $in->child_add_string("objectname", $obj_name);
        $out = $s->invoke_elem($in);

        if($out->results_status() eq "failed")
        {
                print($out->results_reason() ."\n");
                exit(-2);
        }

        my $instances_list = $out->child_get("instances");
        my @instances = $instances_list->children_get();

        foreach $inst (@instances){
                my $inst_name = $inst->child_get_string("name");
		if ( $inst_name =~ m/:/ ) {             
                        # This remove stuff after ":" in LUN name"
                        $inst_name =~ /(.*) :/;
                        $inst_name = $1;
                }
                my $counters_list = $inst->child_get("counters");
                my @counters =  $counters_list->children_get();
                if ($inst_name eq "$instance_selected")
                {
                        if (! "$counter_selected") {
                                print_counter($inst_name, $inst_name);
                        }
                        foreach $counter (@counters) 
                        {
                                my $counter_name = $counter->child_get_string("name");
                                my $counter_value = $counter->child_get_string("value");
                                if ("$counter_selected")
                                {
                                        if ("$counter_name" eq "$counter_selected")
                                        {
                                                print ("$counter_value");
                                        }
                                } else {
                                        print_counter($counter_name, $counter_value);
                                }
                        }
                } elsif (! "$instance_selected") {
                        if (! "$counter_selected") {
				print_counter('name', $inst_name);
                                foreach $counter (@counters) 
                                {
                                        my $counter_name = $counter->child_get_string("name");
                                        my $counter_value = $counter->child_get_string("value");
                                        print_counter($counter_name, $counter_value);
                                }
				print "\n";
                        } else {
                                foreach $counter (@counters)
                                {
                                        my $counter_name = $counter->child_get_string("name");
                                        my $counter_value = $counter->child_get_string("value");
                                        if ("$counter_name" eq "$counter_selected")
                                        {
                                                print_counter($inst_name, $counter_value);
                                        }
                                }
                        }
                        print("\n");
                }
        }
}



#
#Generic output format used for Cacti compatibility
sub print_counter()
{
        my $counter_name = $_[0];
        my $counter_value = $_[1];
        print("$counter_name:$counter_value ");
}


sub print_usage() 
{

        print "\n$progname <filer> <user> <password> <objname> <operation> <counter> <instance>\n";
        print "\n<filer>     -- Filer name\n";
        print "<user>      -- User name\n";
        print "<password>  -- Password\n";
	print "<objname>   -- Perf Object name\n";
        print "<operation> -- Operation to be performed: \n";
        print "\tobject-list   - Get the list of perforance objects in the system\n";
        print "\tindex - Get the list of instances for a given performance object\n";
        print "\tquery - Get the values of the counters for the instance of a performance object\n";
        print "\tget   - Get the values of the counters for all the instances of a performance object\n";
	print "\tcounter-list  - Get the list of counters available for a given performance object\n";
        print "[<counter>]  -- Performance counter name \n";
        print "[<instance>]  -- Instance name (ex: volume name) \n";
        exit -1;
}

